#ifndef GNUPLOT_H
#define GNUPLOT_H


#include "common.h"
#include "axis.h"



//-----------------------------------------------------------------//
//  PLOT用クラス
//-----------------------------------------------------------------//
//    template <typename T1=int, int T2=3>
class GNUPlot
{
public:



//--------------------------------------------------//
//  変数
//--------------------------------------------------//
public:
    int m_target;        //x
    int m_x_label;       //x
    int m_y_label;       //x
    int m_disp_index;    //x
    int m_diff_index;    //x
    int plot_style;
    int color_map;
    int surface;
    int z_unit;

    boost::multi_array<std::complex<double>, 3>* matrix;  // 配列データ
    std::string title;                                    // グラフのタイトル関連

    // マトリックスの表示対象となる次元のインデックス
    // 必ずZ軸に配列要素を表示する仕様なので、z_indexは [マトリックスの次元数 +1]の固定値である。
    int x_index;
    int y_index;
    int z_index;

    // Z軸の情報も含まれているので、matrixの次元数よりも＋１大きい
    // 添字がmatrixの各次元に対応している。最後の要素はZ軸の情報である。
    std::vector<Axis> axises;



//--------------------------------------------------//
//  コンストラクタ
// このコンストラクタのあとに、必ずsetMatrix()で初期化してから使う事
//--------------------------------------------------//
public:
    //  コンストラクタ1
    GNUPlot()
    {
        qDebug() << __func__ << " : コンストラクタ1";
        plot_style = PlotStyle_UNDEFINE;
        color_map  = ColorMap_UNDEFINE;
        surface    = Surface_UNDEFINE;
        z_unit     = DISP_AMP;
        matrix     = NULL;
    }

    //  コンストラクタ2
    GNUPlot(boost::multi_array<std::complex<double>, 3>* matrix, std::vector<int> indexes)
    {
        qDebug() << __func__  << " : コンストラクタ2";

        setMatrix(matrix, indexes);
        plot_style = PlotStyle_UNDEFINE;
        color_map  = ColorMap_UNDEFINE;
        surface    = Surface_UNDEFINE;
        z_unit     = DISP_AMP;
    }


//--------------------------------------------------//
// メソッド
//--------------------------------------------------//
public:
    int  setMatrix(boost::multi_array<std::complex<double>, 3>* matrix, std::vector<int> indexes);
    void setStyle(int plot_style, int surface, int color_map);
    void setTitle(std::string title);
    void setLabels(std::vector<std::string> labels);
    void setCurrentLabels(std::string x, std::string y, std::string z);
    void setWeights(std::vector<double> weights);
    void setCurrentWeight(double x, double y, double z);
    void setRanges(std::vector<std::pair<std::string, std::string> > ranges);
    void setCurrentRange(std::pair<std::string, std::string> x,std::pair<std::string, std::string> y,std::pair<std::string, std::string> z);
    int  setIndexes(std::vector<int> indexes);
    void setZUnit(int value);
    std::string plot(std::string fullpath);
    std::string plotToTempFile(std::string pathname);
    void plotToFile(std::string fname);
    void exportCSVToFile(std::string fname, bool complex=true);
    void exportCSVToStream(std::ostream& os, bool complex=true);
    void plotToStream(std::ostream& os);
    double calcZValue(int z_unit, std::complex<double>value);
    void ErrorMsg(QString msg);

public:
    ~GNUPlot()
    {
    }
};












#endif // GNUPLOT_H
